@@ -66,6 +66,7 @@ typedef struct {
 	GooFileFormat  format;
 	char          *album;
 	char          *artist;
+	int            year;
 	char          *genre;
 	GList         *tracks;
 	int            tracks_n;
@@ -93,6 +94,7 @@ typedef struct {
 	GtkWidget     *dialog;
 	GtkWidget     *r_progress_progressbar;
 	GtkWidget     *r_progress_label;
+	GtkWidget     *r_track_label;
 } DialogData;
 
 
@@ -220,7 +222,7 @@ create_pipeline (DialogData *data)
 		
 	data->source = gst_element_factory_make ("cdparanoia", "cdparanoia");
 	g_object_set (G_OBJECT (data->source), 
-		      "location", data->location,
+		      "device", data->location,
 		      NULL);
 
 	switch (data->format) {
@@ -258,7 +260,7 @@ create_pipeline (DialogData *data)
 			  G_CALLBACK (encoder_eos_cb), 
 			  data);
 
-	data->sink = gst_element_factory_make ("filesink", "filesink");
+	data->sink = gst_element_factory_make ("gnomevfssink", "filesink");
 
 	gst_bin_add_many (GST_BIN (data->rip_thread), data->source, data->encoder, data->sink, NULL);
 	gst_element_link_many (data->source, data->encoder, data->sink, NULL);
@@ -411,6 +413,7 @@ rip_current_track (DialogData *data)
 	char      *msg;
 	char      *filename;
 	char      *folder;
+	char      *e_title;
 	GstEvent  *event;
 
 	if (data->current_track == NULL) {
@@ -442,10 +445,16 @@ rip_current_track (DialogData *data)
 	msg = g_strdup_printf (_("Extracting track %d of %d: %s"), 
 			       data->current_track_n,
 			       data->tracks_n,
-			       track->title);
+			       "");
 	gtk_label_set_text (GTK_LABEL (data->r_progress_label), msg);
 	g_free (msg);
 
+	e_title = g_markup_escape_text (track->title, -1);
+	msg = g_strconcat ("<b>", e_title, "</b>", NULL);
+	gtk_label_set_markup (GTK_LABEL (data->r_track_label), msg);
+	g_free (e_title);
+	g_free (msg);
+
 	/* Set the filename */
 
 	folder = get_destination_folder (data);
@@ -464,18 +473,33 @@ rip_current_track (DialogData *data)
 
 	/* Set song tags. */
 
-	if (GST_IS_TAG_SETTER (data->encoder)) 
+	if (GST_IS_TAG_SETTER (data->encoder)) {
 		gst_tag_setter_add (GST_TAG_SETTER (data->encoder),   
 				    GST_TAG_MERGE_REPLACE_ALL,
 				    GST_TAG_TITLE, track->title,
 				    GST_TAG_ARTIST, data->artist,
 				    GST_TAG_ALBUM, data->album,
-				    GST_TAG_GENRE, data->genre,
 				    GST_TAG_TRACK_NUMBER, (guint) track->number + 1,
 				    GST_TAG_TRACK_COUNT, (guint) data->total_tracks,
 				    GST_TAG_DURATION, (guint64) track->length * GST_SECOND, 
 				    GST_TAG_COMMENT, _("Ripped with Goobox"),
+				    GST_TAG_ENCODER, PACKAGE_NAME,
+				    GST_TAG_ENCODER_VERSION, PACKAGE_VERSION,
 				    NULL);
+		if (data->genre != NULL)
+			gst_tag_setter_add (GST_TAG_SETTER (data->encoder),   
+					    GST_TAG_MERGE_APPEND,
+					    GST_TAG_GENRE, data->genre,
+					    NULL);
+		if (data->year != 0) {
+			GDate *d = g_date_new_dmy (1, 1, data->year);
+			gst_tag_setter_add (GST_TAG_SETTER (data->encoder),   
+					    GST_TAG_MERGE_APPEND,
+					    GST_TAG_DATE,  g_date_get_julian (d),
+					    NULL);
+			g_date_free (d);
+		}
+	}
 		
 	/* Seek to track. */
 
@@ -531,6 +555,7 @@ dlg_ripper (GooWindow     *window,
 	    GooFileFormat  format,
 	    const char    *album,
 	    const char    *artist,
+	    int            year,
 	    const char    *genre,
 	    int            total_tracks,
 	    GList         *tracks)
@@ -552,7 +577,9 @@ dlg_ripper (GooWindow     *window,
 	data->format = format;
 	data->album = g_strdup (album);
 	data->artist = g_strdup (artist);
-	data->genre = g_strdup (genre);
+	data->year = year;
+	if (genre != NULL)
+		data->genre = g_strdup (genre);
 	data->tracks = track_list_dup (tracks);
 	data->tracks_n = g_list_length (data->tracks);
 	data->total_tracks = total_tracks;
@@ -565,6 +592,7 @@ dlg_ripper (GooWindow     *window,
 	data->dialog = glade_xml_get_widget (data->gui, "ripper_dialog");
 	data->r_progress_progressbar = glade_xml_get_widget (data->gui, "r_progress_progressbar");
 	data->r_progress_label = glade_xml_get_widget (data->gui, "r_progress_label");
+	data->r_track_label = glade_xml_get_widget (data->gui, "r_track_label");
 	btn_cancel = glade_xml_get_widget (data->gui, "r_cancelbutton");
 
 	/* Set widgets data. */
@@ -572,6 +600,8 @@ dlg_ripper (GooWindow     *window,
 #ifdef HAVE_GTK_2_5
 	gtk_label_set_ellipsize (GTK_LABEL (data->r_progress_label),
 				 PANGO_ELLIPSIZE_END);
+	gtk_label_set_ellipsize (GTK_LABEL (data->r_track_label),
+				 PANGO_ELLIPSIZE_END);
 #endif
 
 	/* Set the signals handlers. */
