@@ -41,7 +41,7 @@
 #include "goo-player-cd.h"
 #include "goo-player-info.h"
 #include "goo-window.h"
-#include "goo-volume-button.h"
+#include "goo-volume-tool-button.h"
 #include "gtk-utils.h"
 #include "gtk-file-chooser-preview.h"
 #include "glib-utils.h"
@@ -69,6 +69,8 @@
 #define PLAYER_CHECK_RATE 100
 #define COVER_SIZE 80
 #define IDLE_TIMEOUT 200
+#define FALLBACK_ICON_SIZE 16
+
 
 struct _GooWindowPrivateData {
 	GtkUIManager    *ui;
@@ -77,6 +79,9 @@ struct _GooWindowPrivateData {
 	GtkWidget       *toolbar;
 	GtkWidget       *list_expander;
 
+	WindowSortMethod sort_method;
+	GtkSortType      sort_type;
+
 	GtkWidget       *statusbar;
 	GtkWidget       *file_popup_menu;
 	GtkWidget       *tray_popup_menu;
@@ -179,9 +184,9 @@ window_update_statusbar_list_info (GooWindow *window)
 		const char *album;
 		char        time_text[64];
 		char       *tracks_s = NULL;
-		char       *text;
 		int         year;
-		char       *year_s;
+		char       *year_s = NULL;
+		GString    *status;
 
 		album = goo_player_cd_get_album (GOO_PLAYER_CD (priv->player));
 
@@ -190,27 +195,33 @@ window_update_statusbar_list_info (GooWindow *window)
 
 		year = goo_player_get_year (priv->player);
 		if (year > 0)
-			year_s = g_strdup_printf ("year %d", year);
+			year_s = g_strdup_printf (_("year %d"), year);
                 else
-			year_s = g_strdup ("");
+			year_s = NULL;
 		
-		if (album != NULL)
-			text = g_strconcat (album, 
-					    ", ", year_s, 
-					    ", ", tracks_s, 
-					    ", ", time_text, 
-					    NULL);
-		else
-			text = g_strconcat (year_s, 
-					    ", ", tracks_s, 
-					    ", ", time_text,
-					    NULL);
+		status = g_string_new (NULL);
 
-		gtk_statusbar_push (GTK_STATUSBAR (priv->statusbar), priv->list_info_cid, text);
+		if (album != NULL) {
+			g_string_append (status, album);
+			g_string_append (status, ", ");
+		}
 
+		if (year_s != NULL) {
+			g_string_append (status, year_s);
+			g_string_append (status, ", ");
+		}
+
+		g_string_append (status, tracks_s);
+		g_string_append (status, ", ");
+		g_string_append (status, time_text);
+
+		gtk_statusbar_push (GTK_STATUSBAR (priv->statusbar), 
+				    priv->list_info_cid, 
+				    status->str);
+
+		g_string_free (status, TRUE);
 		g_free (year_s);
 		g_free (tracks_s);
-		g_free (text);
 	}
 }
 
@@ -296,6 +307,87 @@ window_update_sensitivity (GooWindow *window)
 }
 
 
+static GdkPixbuf *
+create_void_icon (GooWindow *window)
+{
+	GtkSettings  *settings;
+	int           width, height, icon_size;
+	GdkPixbuf    *icon;
+
+	settings = gtk_settings_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (window)));
+
+	if (gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU, &width, &height))
+		icon_size = MAX (width, height);
+	else
+		icon_size = FALLBACK_ICON_SIZE;
+
+	icon = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, icon_size, icon_size);
+	gdk_pixbuf_fill (icon, 0x00000000);
+
+	return icon;
+}
+
+
+static gboolean
+get_iter_from_track_number (GooWindow   *window,
+			    int          track_number,
+			    GtkTreeIter *iter)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GtkTreeModel         *model = GTK_TREE_MODEL (priv->list_store);
+
+	if (! gtk_tree_model_get_iter_first (model, iter))
+		return FALSE;
+	
+	do {
+		SongInfo *song;
+		gtk_tree_model_get (model, iter, COLUMN_SONG_INFO, &song, -1);
+		if (song->number == track_number) {
+			song_info_unref (song);
+			return TRUE;
+		}
+		song_info_unref (song);
+	} while (gtk_tree_model_iter_next (model, iter));
+
+	return FALSE;
+}
+
+
+static void
+set_song_icon (GooWindow  *window,
+	       int         track_number,
+	       const char *stock_id)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GtkTreeIter           iter;
+	GdkPixbuf            *icon;
+
+	if (!get_iter_from_track_number (window, track_number, &iter)) 
+		return;
+
+	if (stock_id != NULL)
+		icon = gtk_widget_render_icon (GTK_WIDGET (window), 
+					       stock_id, 
+					       GTK_ICON_SIZE_MENU, 
+					       NULL);
+	else
+		icon = create_void_icon (window);
+	gtk_list_store_set (priv->list_store, &iter,
+			    COLUMN_ICON, icon,
+			    -1);
+	g_object_unref (icon);
+}
+
+
+static void
+set_current_song_icon (GooWindow  *window,
+		       const char *stock_id)
+{
+	if (window->priv->current_song != NULL)
+		set_song_icon (window, window->priv->current_song->number, stock_id);
+}
+
+
 static gboolean
 update_list_idle (gpointer callback_data)
 {
@@ -305,6 +397,7 @@ update_list_idle (gpointer callback_data)
 	GList                *file_list;
 	GList                *scan;
 	int                   i, n = FILES_TO_PROCESS_AT_ONCE;
+	GdkPixbuf            *icon;
 
 	if (priv->update_timeout_handle != 0) {
 		g_source_remove (priv->update_timeout_handle);
@@ -324,6 +417,8 @@ update_list_idle (gpointer callback_data)
 	data->file_list = scan->next;
 	scan->next = NULL;
 
+	icon = create_void_icon (window);
+
 	for (scan = file_list; scan; scan = scan->next) {
 		SongInfo    *song = scan->data;
 		GtkTreeIter  iter;
@@ -335,6 +430,7 @@ update_list_idle (gpointer callback_data)
 		gtk_list_store_set (priv->list_store, &iter,
 				    COLUMN_SONG_INFO, song,
 				    COLUMN_NUMBER, song->number + 1, /*FIXME*/
+				    COLUMN_ICON, icon,
 				    COLUMN_TIME, time_s,
 				    COLUMN_TITLE, song->title,
 				    COLUMN_ARTIST, song->artist,
@@ -342,6 +438,8 @@ update_list_idle (gpointer callback_data)
 		g_free (time_s);
 	}
 
+	g_object_unref (icon);
+
 	if (gtk_events_pending ())
 		gtk_main_iteration_do (TRUE);
 
@@ -356,7 +454,7 @@ update_list_idle (gpointer callback_data)
 void
 goo_window_update (GooWindow *window)
 {
-	goo_player_stop (window->priv->player);
+	goo_window_stop (window);
 	goo_player_update (window->priv->player);
 }
 
@@ -444,13 +542,25 @@ goo_window_update_titles (GooWindow *window)
 		
 		gtk_tree_model_get (model, &iter, COLUMN_SONG_INFO, &song, -1);
 		new_song = find_song_from_number (song_list, song->number);
+		song_info_unref (song);
+
 		if (new_song == NULL)
 			continue;
 
 		gtk_list_store_set (priv->list_store, &iter,
+				    COLUMN_SONG_INFO, new_song,
 				    COLUMN_TITLE, new_song->title,
 				    COLUMN_ARTIST, new_song->artist,
 				    -1);
+
+		/* Update the current song info. */
+		if ((priv->current_song != NULL) 
+		    && (new_song->number == priv->current_song->number)) {
+			song_info_unref (priv->current_song);
+			song_info_ref (new_song);
+			priv->current_song = new_song;
+		}
+
 	} while (gtk_tree_model_iter_next (model, &iter));
 
 	song_list_free (song_list);
@@ -680,6 +790,60 @@ print_playlist (GooWindow *window)
 }
 
 
+static int
+get_track_number_from_position (GooWindow   *window,
+				int          pos)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GtkTreeModel         *model = GTK_TREE_MODEL (priv->list_store);
+	GtkTreeIter           iter;
+	int                   i = 0;
+
+	if (! gtk_tree_model_get_iter_first (model, &iter))
+		return -1;
+	
+	do {
+		SongInfo *song;
+		int       n;
+		gtk_tree_model_get (model, &iter, COLUMN_SONG_INFO, &song, -1);
+		n = song->number;
+		song_info_unref (song);
+		if (i == pos)
+			return n;
+		i++;
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	return -1;
+}
+
+
+static int
+get_position_from_track_number (GooWindow   *window,
+				int          track_number)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GtkTreeModel         *model = GTK_TREE_MODEL (priv->list_store);
+	GtkTreeIter           iter;
+	int                   pos = 0;
+
+	if (! gtk_tree_model_get_iter_first (model, &iter))
+		return -1;
+	
+	do {
+		SongInfo *song;
+		int       n;
+		gtk_tree_model_get (model, &iter, COLUMN_SONG_INFO, &song, -1);
+		n = song->number;
+		song_info_unref (song);
+		if (n == track_number)
+			return pos;
+		pos++;
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	return -1;
+}
+
+
 static void
 create_playlist (GooWindow *window,
 		 gboolean   play_all,
@@ -702,14 +866,15 @@ create_playlist (GooWindow *window,
 	playlist = NULL;
 
 	if (priv->current_song != NULL)
-		pos = priv->current_song->number;
+		pos = get_position_from_track_number (window, priv->current_song->number);
 	n = g_list_length (priv->song_list);
 
 	for (i = 0; i < n; i++, pos = (pos + 1) % n) {
+		int track_number = get_track_number_from_position (window, pos);
 		if ((priv->current_song != NULL) 
-		    && (priv->current_song->number == pos))
+		    && (priv->current_song->number == track_number))
 			continue;
-		playlist = g_list_prepend (playlist, GINT_TO_POINTER (pos));
+		playlist = g_list_prepend (playlist, GINT_TO_POINTER (track_number));
 	}
 
 	playlist = g_list_reverse (playlist);
@@ -721,21 +886,29 @@ create_playlist (GooWindow *window,
 
 		while (playlist != NULL) {
 			GList *item;
-
 			pos = g_rand_int_range (grand, 0, len--);
 			item = g_list_nth (playlist, pos);
 			playlist = g_list_remove_link (playlist, item);
 			random_list = g_list_concat (random_list, item);
 		}
-		g_rand_free (grand);
-
 		playlist = random_list;
+
+		g_rand_free (grand);
 	} 
 
 	priv->playlist = playlist;
 }
 
 
+static void
+play_song (GooWindow *window,
+	   gint       track_number)
+{
+	GooWindowPrivateData *priv = window->priv;
+	goo_player_seek_song (priv->player, track_number);
+}
+
+
 static void
 play_next_song_in_playlist (GooWindow *window)
 {
@@ -763,10 +936,10 @@ play_next_song_in_playlist (GooWindow *window)
 	print_playlist (window);
 
 	if (next == NULL)
-		goo_player_stop (priv->player);
+		goo_window_stop (window);
 	else {
 		int pos = GPOINTER_TO_INT (next->data);
-		goo_player_seek_song (priv->player, pos);
+		play_song (window, pos);
 		priv->playlist = g_list_remove_link (priv->playlist, next);
 		g_list_free (next);
 	}
@@ -870,10 +1043,8 @@ goo_window_unrealize (GtkWidget *widget)
 	if (playlist_visible)
 		save_window_size (window);
 
-	/* FIXME
-	preferences_set_sort_method (window->sort_method);
-	preferences_set_sort_type (window->sort_type);
-	*/
+	preferences_set_sort_method (priv->sort_method);
+	preferences_set_sort_type (priv->sort_type);
 
 	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
@@ -898,7 +1069,10 @@ goo_window_show (GtkWidget *widget)
 	GooWindow *window = GOO_WINDOW (widget);
 	gboolean   view_foobar;
 
-	GTK_WIDGET_CLASS (parent_class)->show (widget);
+	if (! HideShow) 
+		GTK_WIDGET_CLASS (parent_class)->show (widget);
+	else
+		HideShow = FALSE;
 
 	view_foobar = eel_gconf_get_boolean (PREF_UI_TOOLBAR, TRUE);
 	set_active (window, "ViewToolbar", view_foobar);
@@ -928,7 +1102,6 @@ goo_window_class_init (GooWindowClass *class)
 	widget_class->realize   = goo_window_realize;
 	widget_class->unrealize = goo_window_unrealize;
 	widget_class->show      = goo_window_show;
-	widget_class->show      = goo_window_show;
 }
 
 
@@ -964,7 +1137,7 @@ sort_by_name (const char *s1,
 		return 1;
 	else if (s2 == NULL)
 		return -1;
-	else
+	else 
 		return g_utf8_collate (s1, s2);
 }
 
@@ -976,16 +1149,24 @@ title_column_sort_func (GtkTreeModel *model,
 			gpointer      user_data)
 {
 	SongInfo *song1 = NULL, *song2 = NULL;
+	int       retval = -1;
+
         gtk_tree_model_get (model, a, COLUMN_SONG_INFO, &song1, -1);
         gtk_tree_model_get (model, b, COLUMN_SONG_INFO, &song2, -1);
+
 	if ((song1 == NULL) && (song2 == NULL))
-		return 0;
+		retval = 0;
 	else if (song1 == NULL)
-		return 1;
+		retval = 1;
 	else if (song2 == NULL)
-		return -1;
+		retval = -1;
 	else
-		return sort_by_name (song1->title, song2->title);
+		retval = sort_by_name (song1->title, song2->title);
+
+	song_info_unref (song1);
+	song_info_unref (song2);
+
+	return retval;
 }
 
 
@@ -996,20 +1177,29 @@ artist_column_sort_func (GtkTreeModel *model,
 			 gpointer      user_data)
 {
 	SongInfo *song1 = NULL, *song2 = NULL;
+	int       retval = -1;
+
         gtk_tree_model_get (model, a, COLUMN_SONG_INFO, &song1, -1);
         gtk_tree_model_get (model, b, COLUMN_SONG_INFO, &song2, -1);
+
 	if ((song1 == NULL) && (song2 == NULL))
-		return 0;
+		retval = 0;
 	else if (song1 == NULL)
-		return 1;
+		retval = 1;
 	else if (song2 == NULL)
-		return -1;
+		retval = -1;
 	else {
 		int result = sort_by_name (song1->artist, song2->artist);
 		if (result == 0)
-			return sort_by_number (song1->number, song2->number);
-		return result;
+			retval = sort_by_number (song1->number, song2->number);
+		else
+			retval = result;
 	}
+
+	song_info_unref (song1);
+	song_info_unref (song2);
+
+	return retval;
 }
 
 
@@ -1020,16 +1210,24 @@ time_column_sort_func (GtkTreeModel *model,
 		       gpointer      user_data)
 {
 	SongInfo *song1 = NULL, *song2 = NULL;
+	int       retval = -1;
+
         gtk_tree_model_get (model, a, COLUMN_SONG_INFO, &song1, -1);
         gtk_tree_model_get (model, b, COLUMN_SONG_INFO, &song2, -1);
+
 	if ((song1 == NULL) && (song2 == NULL))
-		return 0;
+		retval = 0;
 	else if (song1 == NULL)
-		return 1;
+		retval = 1;
 	else if (song2 == NULL)
-		return -1;
+		retval = -1;
 	else 
-		return sort_by_number (song1->time, song2->time);
+		retval = sort_by_number (song1->time, song2->time);
+
+	song_info_unref (song1);
+	song_info_unref (song2);
+
+	return retval;
 }
 
 
@@ -1150,7 +1348,7 @@ goo_window_select_current_song (GooWindow *window)
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->list_view));
 	gtk_tree_selection_unselect_all (selection);
 
-	pos = priv->current_song->number;
+	pos = get_position_from_track_number (window, priv->current_song->number);
 	if (pos == -1)
 		return;
 
@@ -1185,6 +1383,7 @@ goo_window_set_current_song (GooWindow *window,
 	SongInfo *song;
 
 	if (priv->current_song != NULL) {
+		set_current_song_icon (window, NULL);
 		song_info_unref (priv->current_song);	
 		priv->current_song = NULL;
 	}
@@ -1324,9 +1523,10 @@ player_done_cb (GooPlayer       *player,
 	switch (action) {
 	case GOO_PLAYER_ACTION_LIST:
 		goo_player_info_update_state (GOO_PLAYER_INFO (priv->info));
-		window_update_title (window);
 		goo_window_update_list (window);
 		goo_window_update_cover (window);
+		window_update_title (window);
+		set_current_song_icon (window, NULL);
 		if (AutoPlay) {
 			AutoPlay = FALSE;
 			g_idle_add (autoplay_cb, window);
@@ -1334,12 +1534,14 @@ player_done_cb (GooPlayer       *player,
 		break;
 	case GOO_PLAYER_ACTION_METADATA:
 		goo_player_info_update_state (GOO_PLAYER_INFO (priv->info));
-		window_update_title (window);
 		goo_window_update_titles (window);
+		window_update_title (window);
+		window_update_statusbar_list_info (window);
 		break;
 	case GOO_PLAYER_ACTION_SEEK_SONG:
 		goo_window_set_current_song (window, goo_player_get_current_song (priv->player));
 		goo_window_select_current_song (window);
+		set_current_song_icon (window, GOO_STOCK_PLAY);
 		break;
 	case GOO_PLAYER_ACTION_PLAY:
 	case GOO_PLAYER_ACTION_STOP:
@@ -1351,11 +1553,16 @@ player_done_cb (GooPlayer       *player,
 					   GOO_STOCK_PLAY,
 					   "/MenuBar/CDMenu/",
 					   NULL);
-		if (action == GOO_PLAYER_ACTION_PLAY)
+		if (action == GOO_PLAYER_ACTION_PLAY) {
+			set_current_song_icon (window, GOO_STOCK_PLAY);
 			priv->next_timeout_handle = g_timeout_add (IDLE_TIMEOUT, next_time_idle, window);
+		} else if (action == GOO_PLAYER_ACTION_STOP) {
+			set_current_song_icon (window, GOO_STOCK_STOP);
+		}
 
 		break;
 	case GOO_PLAYER_ACTION_PAUSE:
+		set_current_song_icon (window, GOO_STOCK_PAUSE);
 		set_action_label_and_icon (window,
 					   "TogglePlay", 
 					   _("_Play"), 
@@ -1431,8 +1638,11 @@ row_activated_cb (GtkTreeView       *tree_view,
 			    COLUMN_SONG_INFO, &song,
 			    -1);
 
+	goo_window_stop (window);
 	goo_window_set_current_song (window, song->number);
 	goo_window_play (window);
+
+	song_info_unref (song);
 }
 
 
@@ -1504,6 +1714,68 @@ file_button_press_cb (GtkWidget      *widget,
 }
 
 
+static int
+get_column_from_sort_method (WindowSortMethod sort_method)
+{
+	switch (sort_method) {
+	case WINDOW_SORT_BY_NUMBER: return COLUMN_NUMBER;
+	case WINDOW_SORT_BY_TIME: return COLUMN_TIME;
+	case WINDOW_SORT_BY_TITLE: return COLUMN_TITLE;
+	default: 
+		break;
+	}
+
+	return COLUMN_NUMBER;
+}
+
+
+static int
+get_sort_method_from_column (int column_id)
+{
+	switch (column_id) {
+	case COLUMN_NUMBER: return WINDOW_SORT_BY_NUMBER;
+	case COLUMN_TIME: return WINDOW_SORT_BY_TIME;
+	case COLUMN_TITLE: return WINDOW_SORT_BY_TITLE;
+	default: 
+		break;
+	}
+
+	return WINDOW_SORT_BY_NUMBER;
+}
+
+
+static void
+sort_column_changed_cb (GtkTreeSortable *sortable,
+			gpointer         user_data)
+{
+	GooWindow *window = user_data;
+	GooWindowPrivateData  *priv = window->priv;
+	GtkSortType    order;
+	int            column_id;
+	GooPlayerState state;
+
+	if (! gtk_tree_sortable_get_sort_column_id (sortable, 
+						    &column_id, 
+						    &order))
+		return;
+
+	priv->sort_method = get_sort_method_from_column (column_id);
+	priv->sort_type = order;
+
+	state = goo_player_get_state (priv->player);
+
+	/* Recreate the playlist if not already playing. */
+	if ((state != GOO_PLAYER_STATE_PLAYING) &&  (state != GOO_PLAYER_STATE_PAUSED)) { 
+		gboolean   play_all;
+		gboolean   shuffle;
+		
+		play_all = eel_gconf_get_boolean (PREF_PLAYLIST_PLAYALL, TRUE);
+		shuffle = eel_gconf_get_boolean (PREF_PLAYLIST_SHUFFLE, FALSE);
+		create_playlist (window, play_all, shuffle);
+	}
+}
+
+
 static void
 update_ui_from_expander_state (GooWindow *window)
 {
@@ -1516,13 +1788,13 @@ update_ui_from_expander_state (GooWindow *window)
 			gtk_window_resize (GTK_WINDOW (window), 
 					   eel_gconf_get_integer (PREF_UI_WINDOW_WIDTH, DEFAULT_WIN_WIDTH),
 					   eel_gconf_get_integer (PREF_UI_WINDOW_HEIGHT, DEFAULT_WIN_HEIGHT));
-		/*gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (priv->statusbar), TRUE); FIXME*/
+		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (priv->statusbar), TRUE);
 		gtk_widget_show (priv->list_view->parent);
 		gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
 	} else {
 		if (GTK_WIDGET_REALIZED (window))
 			save_window_size (window);
-		/*gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (priv->statusbar), FALSE); FIXME*/
+		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (priv->statusbar), FALSE); 
 		gtk_expander_set_label (expander, _(SHOW_TRACK_LIST));
 		gtk_widget_hide (priv->list_view->parent);
 		gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
@@ -1609,6 +1881,7 @@ window_key_press_cb (GtkWidget   *widget,
 	}
 
 	if (new_song >= 0 && new_song <= window->priv->songs - 1) {
+		goo_window_stop (window);
 		goo_window_set_current_song (window, new_song);
 		goo_window_play (window);
 	}
@@ -1693,10 +1966,10 @@ tray_icon_expose (GtkWidget      *widget,
 
 
 static void
-volume_button_changed_cb (GooVolumeButton *button,
-			  GooWindow       *window)
+volume_button_changed_cb (GooVolumeToolButton *button,
+			  GooWindow           *window)
 {
-	int vol = (int) goo_volume_button_get_volume (button);
+	int vol = (int) goo_volume_tool_button_get_volume (button);
 	goo_player_set_volume (window->priv->player, vol);
 }
 
@@ -1720,7 +1993,6 @@ goo_window_construct (GooWindow  *window,
 	GtkWidget            *scrolled_window;
 	GtkWidget            *vbox;
 	GtkWidget            *hbox;
-	GtkWidget            *volume_box;
 	GtkWidget            *expander;
 	GtkTreeSelection     *selection;
 	int                   i;
@@ -1757,7 +2029,7 @@ goo_window_construct (GooWindow  *window,
 	/* * File list. */
 
 	priv->list_store = gtk_list_store_new (NUMBER_OF_COLUMNS, 
-					       G_TYPE_POINTER,
+					       GOO_TYPE_SONG_INFO,
 					       G_TYPE_INT,
 					       GDK_TYPE_PIXBUF,
 					       G_TYPE_STRING,
@@ -1782,12 +2054,11 @@ goo_window_construct (GooWindow  *window,
 					 COLUMN_TIME, time_column_sort_func,
 					 NULL, NULL);
 
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (priv->list_store), COLUMN_NUMBER, GTK_SORT_ASCENDING);
-
-	/*
-	gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (window->list_store), default_sort_func,  NULL, NULL);
-	*/
+	priv->sort_method = preferences_get_sort_method ();
+	priv->sort_type = preferences_get_sort_type ();
 
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (priv->list_store), get_column_from_sort_method (priv->sort_method), priv->sort_type);
+	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->list_view));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
@@ -1804,14 +2075,14 @@ goo_window_construct (GooWindow  *window,
 			  G_CALLBACK (file_button_press_cb), 
 			  window);
 
-	/*
-	egg_tree_multi_drag_add_drag_support (GTK_TREE_VIEW (window->list_view));
-
-	g_signal_connect (G_OBJECT (window->list_store), 
+	g_signal_connect (G_OBJECT (priv->list_store), 
 			  "sort_column_changed",
 			  G_CALLBACK (sort_column_changed_cb), 
 			  window);
 
+	/*
+	egg_tree_multi_drag_add_drag_support (GTK_TREE_VIEW (window->list_view));
+
         gtk_drag_source_set (window->list_view, 
 			     GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
 			     target_table, n_targets, 
@@ -1894,7 +2165,7 @@ goo_window_construct (GooWindow  *window,
 			      1, 1, 0);
 
 	priv->toolbar = toolbar = gtk_ui_manager_get_widget (ui, "/ToolBar");
-	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), TRUE);
+	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);
 	gnome_app_add_docked (GNOME_APP (window),
 			      toolbar,
 			      "ToolBar",
@@ -1903,11 +2174,50 @@ goo_window_construct (GooWindow  *window,
 			       | (eel_gconf_get_boolean (PREF_DESKTOP_TOOLBAR_DETACHABLE, TRUE) ? BONOBO_DOCK_ITEM_BEH_NORMAL : BONOBO_DOCK_ITEM_BEH_LOCKED)),
 			      BONOBO_DOCK_TOP,
 			      2, 1, 0);
-	/*gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS); FIXME*/
+
+	{
+		GtkAction *action;
+
+		action = gtk_ui_manager_get_action (ui, "/ToolBar/Play");
+		g_object_set (action, "is_important", TRUE, NULL);
+		g_object_unref (action);
+
+		action = gtk_ui_manager_get_action (ui, "/ToolBar/Pause");
+		g_object_set (action, "is_important", TRUE, NULL);
+		g_object_unref (action);
+
+
+		action = gtk_ui_manager_get_action (ui, "/ToolBar/Stop");
+		g_object_set (action, "is_important", TRUE, NULL);
+		g_object_unref (action);
+	}
+
 
 	priv->file_popup_menu = gtk_ui_manager_get_widget (ui, "/ListPopupMenu");
 	priv->cover_popup_menu = gtk_ui_manager_get_widget (ui, "/CoverPopupMenu");
 
+	/* Add the volume button to the toolbar. */
+	
+	{
+		GtkToolItem *sep = gtk_separator_tool_item_new ();
+		
+		gtk_widget_show (GTK_WIDGET (sep));
+		gtk_toolbar_insert (GTK_TOOLBAR (priv->toolbar), 
+				    GTK_TOOL_ITEM (sep),
+				    -1);
+	}
+
+	priv->volume_button = (GtkWidget*) goo_volume_tool_button_new (0.0, 100.0, 5.0);
+	g_signal_connect (priv->volume_button, 
+			  "changed",
+			  G_CALLBACK (volume_button_changed_cb), 
+			  window);
+	gtk_widget_show (GTK_WIDGET (priv->volume_button));
+	gtk_tool_item_set_is_important (GTK_TOOL_ITEM (priv->volume_button), FALSE); /*FIXME*/
+	gtk_toolbar_insert (GTK_TOOLBAR (priv->toolbar), 
+			    GTK_TOOL_ITEM (priv->volume_button), 
+			    -1);
+
 	/* Create the statusbar. */
 
 	priv->statusbar = gtk_statusbar_new ();
@@ -1920,14 +2230,15 @@ goo_window_construct (GooWindow  *window,
 
 	/**/
 
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
 
-	hbox = gtk_hbox_new (FALSE, 12);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	priv->info = goo_player_info_new (NULL);
+	priv->info = goo_player_info_new (window, NULL);
+	gtk_container_set_border_width (GTK_CONTAINER (priv->info), 0);
 	g_signal_connect (priv->info,
 			  "skip_to",
 			  G_CALLBACK (player_info_skip_to_cb), 
@@ -1940,20 +2251,6 @@ goo_window_construct (GooWindow  *window,
 
 	/**/
 
-	volume_box = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (volume_box), 0);
-	gtk_box_pack_start (GTK_BOX (hbox), volume_box, FALSE, FALSE, 0);
-
-	priv->volume_button = goo_volume_button_new (0.0, 100.0, 5.0);
-	gtk_box_pack_start (GTK_BOX (volume_box), priv->volume_button, FALSE, FALSE, 0);
-
-	g_signal_connect (priv->volume_button, 
-			  "changed",
-			  G_CALLBACK (volume_button_changed_cb), 
-			  window);
-
-	/**/
-
 	priv->list_expander = expander = gtk_expander_new_with_mnemonic (_(HIDE_TRACK_LIST));
 	gtk_expander_set_expanded (GTK_EXPANDER (expander), eel_gconf_get_boolean (PREF_UI_PLAYLIST, TRUE));
 	g_signal_connect (expander,
@@ -1963,7 +2260,7 @@ goo_window_construct (GooWindow  *window,
 
 	gtk_box_pack_start (GTK_BOX (vbox), 
 			    expander, 
-			    FALSE, FALSE, 0);
+			    FALSE, FALSE, 6);
 
 	gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0);
 
@@ -2010,9 +2307,9 @@ goo_window_construct (GooWindow  *window,
 
 	goo_player_info_set_player (GOO_PLAYER_INFO (priv->info), priv->player);
 
-	goo_volume_button_set_volume (GOO_VOLUME_BUTTON (priv->volume_button), 
-				      eel_gconf_get_integer (PREF_GENERAL_VOLUME, DEFAULT_VOLUME),
-				      TRUE);
+	goo_volume_tool_button_set_volume (GOO_VOLUME_TOOL_BUTTON (priv->volume_button), 
+					   eel_gconf_get_integer (PREF_GENERAL_VOLUME, DEFAULT_VOLUME),
+					   TRUE);
 
 	/* Create the tray icon. */
 	
@@ -2210,6 +2507,9 @@ goo_window_play (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
 
+	if (goo_player_get_state (priv->player) == GOO_PLAYER_STATE_PLAYING) 
+		return;
+
 	if (goo_player_get_state (priv->player) != GOO_PLAYER_STATE_PAUSED) {
 		gboolean  play_all;
 		gboolean  shuffle;
@@ -2219,13 +2519,15 @@ goo_window_play (GooWindow *window)
 		create_playlist (window, play_all, shuffle);
 
 		if (priv->current_song != NULL) 
-			goo_player_seek_song (priv->player, priv->current_song->number);
+			play_song (window, priv->current_song->number);
 		else if (window->priv->playlist != NULL)
 			play_next_song_in_playlist (window);
 		else
-			goo_player_seek_song (priv->player, 0);
-	} else
+			play_song (window, 0);
+	} else {
+		set_current_song_icon (window, GOO_STOCK_PLAY);
 		goo_player_play (priv->player);
+	}
 }
 
 
@@ -2238,6 +2540,7 @@ goo_window_play_selected (GooWindow *window)
 
 	if (g_list_length (song_list) == 1) {
 		SongInfo * song = song_list->data;
+		goo_window_stop (window);
 		goo_window_set_current_song (window, song->number);
 		goo_window_play (window);
 	}
@@ -2283,9 +2586,17 @@ goo_window_next (GooWindow *window)
 
 	if (priv->current_song != NULL) {
 		if (window->priv->playlist == NULL) {
-			int n = MIN (priv->current_song->number + 1, priv->songs - 1);
+			int current_track = priv->current_song->number;
+			int current_pos;
+			int new_pos, new_track;
+
 			goo_window_stop (window);
-			goo_window_set_current_song (window, n);
+
+			current_pos = get_position_from_track_number (window, current_track);
+			new_pos = MIN (current_pos + 1, priv->songs - 1);
+			new_track = get_track_number_from_position (window, new_pos);
+			goo_window_set_current_song (window, new_track);
+
 			goo_window_play (window);
 		} else
 			play_next_song_in_playlist (window);
@@ -2301,18 +2612,24 @@ void
 goo_window_prev (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
-	int n;
+	int new_pos, new_track;
 
 	if (priv->songs == 0)
 		return;
 
-	if (priv->current_song != NULL)
-		n = MAX ((gint)priv->current_song->number - 1, 0);
-	else
-		n = priv->songs - 1;
-
 	goo_window_stop (window);
-	goo_window_set_current_song (window, n);
+
+	if (priv->current_song != NULL) {
+		int current_track, current_pos;
+		current_track = priv->current_song->number;
+		current_pos = get_position_from_track_number (window, current_track);
+		new_pos = MAX (current_pos - 1, 0);
+	} else
+		new_pos = priv->songs - 1;
+
+	new_track = get_track_number_from_position (window, new_pos);
+	goo_window_set_current_song (window, new_track);
+
 	goo_window_play (window);
 }
 
@@ -2352,7 +2669,6 @@ add_selected_song (GtkTreeModel *model,
 	gtk_tree_model_get (model, iter,
 			    COLUMN_SONG_INFO, &song,
 			    -1);
-	song_info_ref (song);
 	*list = g_list_prepend (*list, song);
 }
 
@@ -2683,7 +2999,7 @@ goo_window_toggle_visibility (GooWindow *window)
 					   _("_Show Window"), 
 					   _("Show the main window"),
 					   NULL,
-					   "/MenuBar/View/",
+					   /*"/MenuBar/View/",  FIXME*/
 					   "/TrayPopupMenu/",
 					   NULL);
 
@@ -2697,8 +3013,30 @@ goo_window_toggle_visibility (GooWindow *window)
 					   _("_Hide Window"), 
 					   _("Hide the main window"),
 					   NULL,
-					   "/MenuBar/View/",
+					   /*"/MenuBar/View/", FIXME*/
 					   "/TrayPopupMenu/",
 					   NULL);
 	}
 }
+
+
+double
+goo_window_get_volume (GooWindow   *window)
+{
+	GooVolumeToolButton *volume_button;
+
+	volume_button = GOO_VOLUME_TOOL_BUTTON (window->priv->volume_button);
+	return goo_volume_tool_button_get_volume (volume_button);
+}
+
+
+void
+goo_window_set_volume (GooWindow   *window,
+		       double       value)
+{
+	GooVolumeToolButton *volume_button;
+
+	volume_button = GOO_VOLUME_TOOL_BUTTON (window->priv->volume_button);
+	goo_volume_tool_button_set_volume (volume_button, value, TRUE);
+}
+
