@@ -41,10 +41,13 @@
 #define PREF_PLAYLIST_PLAYALL         "/apps/goobox/playlist/play_all"
 #define PREF_PLAYLIST_SHUFFLE         "/apps/goobox/playlist/shuffle"
 #define PREF_PLAYLIST_REPEAT          "/apps/goobox/playlist/repeat"
+#define PREF_PLAYLIST_SORT_METHOD     "/apps/goobox/playlist/sort_method"
+#define PREF_PLAYLIST_SORT_TYPE       "/apps/goobox/playlist/sort_type"
 
 #define PREF_EXTRACT_DESTINATION      "/apps/goobox/dialogs/extract/destination"
 #define PREF_EXTRACT_FILETYPE         "/apps/goobox/dialogs/extract/file_type"
 #define PREF_EXTRACT_SAVE_PLAYLIST    "/apps/goobox/dialogs/extract/save_playlist"
+#define PREF_EXTRACT_FIRST_TIME       "/apps/goobox/dialogs/extract/first_time"
 #define PREF_RIPPER_VIEW_DISTINATION  "/apps/goobox/dialogs/ripper/view_destination"
 
 #define PREF_ENCODER_OGG_QUALITY      "/apps/goobox/encoder/ogg_quality"
@@ -65,4 +68,12 @@ void                pref_util_restore_window_geometry (GtkWindow  *window,
 GooFileFormat       pref_get_file_format              (void);
 void                pref_set_file_format              (GooFileFormat value);
 
+
+WindowSortMethod    preferences_get_sort_method       (void);
+void                preferences_set_sort_method       (WindowSortMethod i_value);
+
+GtkSortType         preferences_get_sort_type         (void);
+void                preferences_set_sort_type         (GtkSortType i_value);
+
+
 #endif /* PREFERENCES_H */
